
(function() {
  tinymce.create('tinymce.plugins.PP_Editor_Image', {
    /**
     * Initializes the plugin, this will be executed after the plugin has been created.
     * This call is done before the editor instance has finished it's initialization so use the onInit event
     * of the editor instance to intercept that event.
     *
     * @param {tinymce.Editor} ed Editor instance that the plugin is initialized in.
     * @param {string} url Absolute URL to where the plugin is located.
     */
    init : function(ed, url) {
      var t = this;

      // Register the command so that it can be invoked by using tinyMCE.activeEditor.execCommand();
      ed.addCommand('mcePPEdImage', function() {
        var elem = ed.selection.getNode();
        // Look up three level of parents for <div mce_name=pp_img> parent node
        for (i = 0; i < 3; i++) {
          if ($(elem).attr('mce_name') == 'pp_img')
            break;
          
          elem = $(elem).parent();
        }
        
        if ($(elem).attr('mce_name') == 'pp_img') {
          // We have selected an existing pp_img
          var type = $(elem).attr('type');
          if (type == 'node') {
            nid = $(elem).attr('nid');
            url = Drupal.settings.basePath + '?q=pp_ed/insert/ifimage/insert/' + nid;
            query = '&preset=' + $(elem).attr('preset');
          }
          else if (type == 'url') {
            url = Drupal.settings.basePath + '?q=pp_ed/insert/ifimage/url';
            query = '&src=' + $(elem).attr('src') +
                    '&height=' + $(elem).attr('height') +
                    '&width=' + $(elem).attr('width');
          }

          // deliberate assignment, not conditional equals
          if (title = $(elem).attr('title')) {
            query += '&title=' + title;
          }
          if (caption = $(elem).attr('caption')) {
            query += '&caption=' + caption;
          }
          if (align = $(elem).attr('align')) {
            query += '&align=' + align;
          }
          if (link = $(elem).attr('link')) {
            query += '&link=' + link;
          }
          if (target = $(elem).attr('target')) {
            query += '&target=' + target;
          }
          if (teaser = $(elem).attr('teaser')) {
            query += '&teaser=' + teaser;
          }

          if (url && query) {
            ed.windowManager.open({
              file : url + query,
              width : 800,
              height : 500,
              scrollbars : 'yes'
            });
          }
        }
        else {
          // No we haven't. New image. 
          ed.windowManager.open({
            file : Drupal.settings.basePath + '?q=pp_ed/insert/ifimage/start',
            width : 800,
            height : 500,
            scrollbars : 'yes'
          });
        }
      });

      // Register the command so that it can be invoked by using tinyMCE.activeEditor.execCommand();
      ed.addCommand('mcePPEdImageDelete', function() {
        var elem = ed.selection.getNode();
        // Look up three level of parents for <div mce_name=pp_img> parent node
        for (i = 0; i < 3; i++) {
          if ($(elem).attr('mce_name') == 'pp_img')
            break;
          
          elem = $(elem).parent();
        }
        
        if ($(elem).attr('mce_name') == 'pp_img') {
          ed.dom.remove(elem);
          ed.execCommand('mceRepaint');
        }
      });

      // Register image button
      ed.addButton('pp_ed_image', {
        title : 'ProsePoint Image Insert',
        cmd : 'mcePPEdImage',
        image : url + '/img/tree.gif'
      });

      // Register image delete button
      ed.addButton('pp_ed_image_delete', {
        title : 'Delete Image',
        cmd : 'mcePPEdImageDelete',
        image : url + '/img/img_del.gif'
      });

      ed.onNodeChange.add(function(ed, cm, n) {
        elem = n;
        // Look up three level of parents for <div mce_name=pp_img> parent node
        for (i = 0; i < 3; i++) {
          if ($(elem).attr('mce_name') == 'pp_img')
            break;
          
          elem = $(elem).parent();
        }
        
        if ($(elem).attr('mce_name') == 'pp_img') {
          cm.setActive('pp_ed_image', true);
          cm.setActive('pp_ed_image_delete', true);
          cm.setDisabled('pp_ed_image_delete', false);
        } else {
          cm.setActive('pp_ed_image', false);
          cm.setActive('pp_ed_image_delete', false);
          cm.setDisabled('pp_ed_image_delete', true);
        }
      });
      
      ed.onBeforeSetContent.add(function(ed, o) {
        o.content = t.__cleanup(o.content);
        o.content = t.__to_html(o.content);
      });
      ed.onGetContent.add(function(ed, o) {
        o.content = t.__from_html(o.content);
      });

      /*
      ed.onSetContent.add(function(ed, o) {
        alert('onSetContent: ' + o.content);
      });
      ed.onBeforeGetContent.add(function(ed, o) {
        alert('onBeforeGetContent: ' + o.content);
      });
      ed.onPreProcess.add(function(ed, o) {
        alert('onPreProcess: ' + o.content);
      });
      ed.onPostProcess.add(function(ed, o) {
        if (o.get) {
          alert('onPostProcess: get: ' + o.content);
          // o.content = t.__from_html(o.content);
        }
        else if (o.set)
          alert('onPostProcess: set: ' + o.content);
        else
          alert('onPostProcess: ??');
      });
      */
    },

    onInit : function(ed) {
      ed.controlManager.setDisabled('pp_ed_image_delete', TRUE);
    },

    // Cleanups 
    __cleanup : function(co) {
      // Move <pp_img> tag outside of <p>'s
      var data = co.replace(/<p\s*>(.|\n)*?<\/p\s*>/gi, function (para) {
        var data2 = para.replace(/(<p\s*>)((?:.|\n)*)(<pp_img.*?<\/pp_img *>)((?:.|\n)*)(<\/p\s*>)/gi, function (unused, p_start, preamble, pp_elem, postamble, p_end) {
          // Trim whitespace from both sides
          preamble = preamble.replace(/^\s*|\s*$/, '');
          postamble = postamble.replace(/^\s*|\s*$/, '');
          
          if (preamble == '' && postamble == '') {
            output = pp_elem;
          }
          else if (preamble == '') {
            output = pp_elem + p_start + postamble + p_end;
          }
          else if (postamble == '') {
            output = p_start + preamble + p_end + pp_elem;
          }
          else {
            output = p_start + preamble + p_end + pp_elem + p_start + postamble + p_end;            
          }
          
          return output;
        });
        return data2;
      });
      return data;
    },

    // convert internal notation to html
    __to_html : function(co) {
      // alert('__to_html: input: ' + co);       
      var data = co.replace(/(<pp_img.*?)<\/pp_img *>/gi, function (a, b) {
        var html = $.ajax({url: Drupal.settings.basePath + '?q=pp_ed/render/pp_img' + '&pp_img=' + encodeURIComponent(b.substring(1,b.length-2)), async: false }).responseText;
        var output = b + html + ' </pp_img>';   // A space prevents tinymce from trying to clean up this tag
        return output;
      });
      // alert('__to_html: output: ' + data);
      return data;
    },

    // convert html to internal notation
    __from_html : function(co) {
      // alert('__from_html: input: ' + co);
      var data = co.replace(/(<pp_img[^>]*>)(.|\n)+?<\/pp_img *>/gi, function (a, b) {
        var result = /<p class="caption">((.|\n)+?)<\/p>/i.exec(a);
        // Move user-editable caption over into pp_img tag
        if (result && result[1]) {
          result = result[1];
          b = b.replace(/caption=".+?"/i, 'caption="' + result + '"');          
        }
        return b + ' </pp_img>';
      });
      // alert('__from_html: output: ' + data);
      return data;
    },

    // convert internal notation to html, but as an already transformed tinymce custom element
    __to_html_for_insert : function(co) {
      // alert('__to_html_for_insert: input: ' + co);       
      var data = co.replace(/(<pp_img(.*?))<\/pp_img *>/gi, function (a, b, c) {
        var html = $.ajax({url: Drupal.settings.basePath + '?q=pp_ed/render/pp_img' + '&pp_img=' + encodeURIComponent(b.substring(1,b.length-2)), async: false }).responseText;
        var output = '<div mce_name="pp_img"' + c + html + ' </div>';   // A space prevents tinymce from trying to clean up this tag
        return output;
      });
      // alert('__to_html_for_insert: output: ' + data);
      return data;
    },


    /**
     * Creates control instances based in the incomming name. This method is normally not
     * needed since the addButton method of the tinymce.Editor class is a more easy way of adding buttons
     * but you sometimes need to create more complex controls like listboxes, split buttons etc then this
     * method can be used to create those.
     *
     * @param {String} n Name of the control to create.
     * @param {tinymce.ControlManager} cm Control manager to use inorder to create new control.
     * @return {tinymce.ui.Control} New control instance or null if no control was created.
     */
    createControl : function(n, cm) {
      return null;
    },

    /**
     * Returns information about the plugin as a name/value array.
     * The current keys are longname, author, authorurl, infourl and version.
     *
     * @return {Object} Name/value array containing information about the plugin.
     */
    getInfo : function() {
      return {
        longname : 'ProsePoint Editor Image Insert',
        author : 'Beng Tan',
        authorurl : 'http://thinkleft.com.au',
        infourl : 'http://www.prosepoint.org',
        version : "0.03"
      };
    }
  });

  // Register plugin
  tinymce.PluginManager.add('pp_ed_image', tinymce.plugins.PP_Editor_Image);
})();